package io.reisub.openosrs.agility;

import com.google.inject.Provides;
import io.reisub.openosrs.agility.tasks.Alch;
import io.reisub.openosrs.agility.tasks.HandleObstacle;
import io.reisub.openosrs.agility.tasks.PickupMark;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Eat;
import io.reisub.openosrs.util.tasks.KittenTask;
import io.reisub.openosrs.util.tasks.Run;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.*;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Agility",
	description = "Hippity hoppity, jumps on your property",
	enabledByDefault = false
)
@Slf4j
public class AgilityPlugin extends iScript {
	@Inject
	private AgilityConfig config;

	private List<Task> tasks;

	private KittenTask kittenTask;
	private Alch alchTask;
	private HandleObstacle handleObstacleTask;
	private PickupMark pickupMarkTask;

	@Provides
	AgilityConfig provideConfig(ConfigManager configManager) {
		return configManager.getConfig(AgilityConfig.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}

		game.sleepDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Agility");

		Eat eatTask = injector.getInstance(Eat.class);
		eatTask.setInterval(14, 24);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		kittenTask = KittenTask.getInstance(injector);
		handleObstacleTask = injector.getInstance(HandleObstacle.class);
		pickupMarkTask = injector.getInstance(PickupMark.class);

		if (config.highAlch()) {
			alchTask = injector.getInstance(Alch.class);
		}

		tasks = new ArrayList<>();
		tasks.add(eatTask);
		tasks.add(runTask);
		tasks.add(kittenTask);
		tasks.add(pickupMarkTask);
		if (config.highAlch()) {
			tasks.add(alchTask);
		}
		tasks.add(handleObstacleTask);

		handleObstacleTask.setReady(true);
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Agility");
		if (tasks != null) {
			tasks.clear();
		}

		KittenTask.handleKitten = false;
	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	@Subscribe
	private void onHitsplatApplied(HitsplatApplied event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onHitsplatApplied(event);
		}

		if (pickupMarkTask != null) {
			pickupMarkTask.onHitsplatApplied(event);
		}
	}

	@Subscribe
	private void onStatChanged(StatChanged event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onStatChanged(event);
		}

		if (pickupMarkTask != null) {
			pickupMarkTask.onStatChanged(event);
		}

		if (alchTask != null) {
			alchTask.onStatChanged(event);
		}
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (handleObstacleTask != null) {
			handleObstacleTask.onGameTick(event);
		}

		if (alchTask != null) {
			alchTask.onGameTick(event);
		}
	}

	@Subscribe
	private void onChatMessage(ChatMessage chatMessage) {
		if (kittenTask != null) {
			kittenTask.onChatMessage(chatMessage);
		}

		if (handleObstacleTask != null) {
			handleObstacleTask.onChatMessage(chatMessage);
		}
	}
}