package io.reisub.openosrs.fisher;

import com.google.inject.Provides;
import io.reisub.openosrs.fisher.tasks.Drop;
import io.reisub.openosrs.fisher.tasks.Fish;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.KittenTask;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Fisher",
	description = "Frantically fishes fish",
	enabledByDefault = false
)
@Slf4j
public class FisherPlugin extends iScript {
	private List<Task> tasks;

	private KittenTask kittenTask;

	@Provides
	FisherConfig provideConfig(ConfigManager configManager) {
		return configManager.getConfig(FisherConfig.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}

		game.sleepDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Fisher");

		kittenTask = KittenTask.getInstance(injector);

		tasks = new ArrayList<>();
		tasks.add(kittenTask);
		tasks.add(injector.getInstance(Drop.class));
		tasks.add(injector.getInstance(Fish.class));
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Fisher");
		if (tasks != null) {
			tasks.clear();
		}

		KittenTask.handleKitten = false;
	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	@Subscribe
	private void onChatMessage(ChatMessage chatMessage) {
		if (kittenTask != null) {
			kittenTask.onChatMessage(chatMessage);
		}
	}
}