package io.reisub.openosrs.shopper;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.api.mixins.Inject;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Shopper",
	description = "Hops worlds and buys stuff from NPCs",
	enabledByDefault = false
)
@Slf4j
public class Shopper extends iScript {
	@Inject
	private Config config;

	private List<Task> tasks;

	@Getter
	private List<Item> items;

	@Getter
	@Setter
	private boolean hop;

	@Provides
	@SuppressWarnings("unused")
    Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}

		game.sleepDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Shopper");

		loadItems();

		tasks = new ArrayList<>();
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Shopper");
		if (tasks != null) {
			tasks.clear();
		}
	}

	@Subscribe
	@SuppressWarnings("unused")
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	private void loadItems() {
		if (config.itemOneEnabled()) {
			items.add(new Item(config.itemOneName(), config.itemOneAmount(), config.itemOneMinInStore()));
		}

		if (config.itemTwoEnabled()) {
			items.add(new Item(config.itemTwoName(), config.itemTwoAmount(), config.itemTwoMinInStore()));
		}

		if (config.itemThreeEnabled()) {
			items.add(new Item(config.itemThreeName(), config.itemThreeAmount(), config.itemThreeMinInStore()));
		}

		if (config.itemFourEnabled()) {
			items.add(new Item(config.itemFourName(), config.itemFourAmount(), config.itemFourMinInStore()));
		}

		if (config.itemFiveEnabled()) {
			items.add(new Item(config.itemFiveName(), config.itemFiveAmount(), config.itemFiveMinInStore()));
		}
	}
}