package io.reisub.openosrs.smelter;

import com.google.inject.Provides;

import io.reisub.openosrs.smelter.tasks.HandleBank;
import io.reisub.openosrs.smelter.tasks.Smelt;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.util.tasks.Eat;
import io.reisub.openosrs.util.tasks.KittenTask;
import io.reisub.openosrs.util.tasks.Run;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.AnimationID;
import net.runelite.api.ChatMessageType;
import net.runelite.api.GameState;
import net.runelite.api.events.AnimationChanged;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

import static net.runelite.api.AnimationID.IDLE;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Smelter",
	description = "Hot stuff",
	enabledByDefault = false
)
@Slf4j
public class Smelter extends iScript {
	private List<Task> tasks;
	private KittenTask kittenTask;

	@Getter
	private Activity currentActivity;

	private Instant lastActionTime;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				log.info(t.getStatus());
				t.execute();
				break;
			}
		}

		checkActionTimeout();
		game.sleepDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Smelter");

		Eat eatTask = injector.getInstance(Eat.class);
		eatTask.setInterval(14, 24);

		Run runTask = injector.getInstance(Run.class);
		runTask.setInterval(70, 95);

		kittenTask = KittenTask.getInstance(injector);

		currentActivity = Activity.IDLE;

		tasks = new ArrayList<>();
		tasks.add(eatTask);
		tasks.add(runTask);
		tasks.add(kittenTask);
		tasks.add(injector.getInstance(HandleBank.class));
		tasks.add(injector.getInstance(Smelt.class));
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Smelter");
		if (tasks != null) {
			tasks.clear();
		}

		KittenTask.handleKitten = false;
	}

	@SuppressWarnings("unused")
	@Subscribe
	private void onAnimationChanged(AnimationChanged event) {
		if (game.client().getGameState() != GameState.LOGGED_IN) return;
		if (event.getActor() != game.client().getLocalPlayer()) return;


		int animId = game.localPlayer().animation();
		switch (animId) {
			case AnimationID.SMITHING_SMELTING:
			case AnimationID.SMITHING_CANNONBALL:
				setActivity(Activity.SMELTING);
				break;
		}
	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}

	@Subscribe
	private void onChatMessage(ChatMessage chatMessage) {
		if (kittenTask != null) {
			kittenTask.onChatMessage(chatMessage);
		}

		if (chatMessage.getType() == ChatMessageType.GAMEMESSAGE) {
			if (chatMessage.getMessage().startsWith("Congratulations, you've just advanced your")) {
				setActivity(Activity.IDLE);
			}
		}
	}

	private void setActivity(Activity action) {
		currentActivity = action;

		if (action != Activity.IDLE) {
			lastActionTime = Instant.now();
		}
	}

	private void checkActionTimeout() {
		if (currentActivity == Activity.IDLE) return;

		int animId = game.localPlayer().animation();
		if (animId != IDLE || lastActionTime == null) return;

		Duration timeout = Duration.ofSeconds(5);
		Duration sinceAction = Duration.between(lastActionTime, Instant.now());

		if (sinceAction.compareTo(timeout) >= 0) {
			setActivity(Activity.IDLE);
		}
	}
}