package io.reisub.openosrs.util;

import io.reisub.openosrs.util.enums.Activity;
import io.reisub.openosrs.util.tasks.KittenTask;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.ChatMessageType;
import net.runelite.api.GameState;
import net.runelite.api.Skill;
import net.runelite.api.events.ChatMessage;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.api.events.GameStateChanged;
import net.runelite.api.events.StatChanged;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.iutils.scripts.iScript;

import javax.inject.Inject;
import java.time.Duration;
import java.time.Instant;
import java.util.*;

import static net.runelite.api.AnimationID.IDLE;

@Slf4j
public abstract class CScript extends iScript {
    @Inject
    protected Calculations calc;

    protected final List<Task> tasks = new ArrayList<>();
    protected KittenTask kittenTask;

    @Getter
    protected Activity currentActivity = Activity.IDLE;

    protected Instant lastLogin = Instant.EPOCH;
    protected Instant lastActionTime = Instant.EPOCH;
    protected Instant lastExperience = Instant.EPOCH;
    protected final Map<Skill, Activity> idleCheckSkills = new HashMap<>();

    @Override
    protected void loop() {
        for (Task t : tasks) {
            if (t.validate()) {
                log.info(t.getStatus());
                t.execute();
                break;
            }
        }

        checkActionTimeout();
        game.sleepDelay();
    }

    @Override
    protected void onStart() {
        log.info("Starting " + this.getName());
        kittenTask = KittenTask.getInstance(injector);
    }

    @Override
    protected void onStop() {
        log.info("Stopping " + this.getName());
        tasks.clear();

        KittenTask.handleKitten = false;
    }

    protected final <T extends Task> void addTask(Class<T> type) {
        tasks.add(injector.getInstance(type));
    }

    @SuppressWarnings("unused")
    @Subscribe
    private void onConfigButtonPressed(ConfigButtonClicked event) {
        String name = this.getName().replaceAll(" ", "").toLowerCase(Locale.ROOT);

        if (event.getGroup().equals(name) && event.getKey().equals("startButton")) {
            execute();
        }
    }

    @SuppressWarnings("unused")
    @Subscribe
    private void onChatMessage(ChatMessage event) {
        if (kittenTask != null) {
            kittenTask.onChatMessage(event);
        }

        if (event.getType() == ChatMessageType.GAMEMESSAGE) {
            if (event.getMessage().startsWith("Congratulations, you've just advanced your")) {
                setActivity(Activity.IDLE);
            }
        }
    }

    @SuppressWarnings("unused")
    @Subscribe
    private void onStatChanged(StatChanged event) {
        for (Skill skill : idleCheckSkills.keySet()) {
            if (event.getSkill() == skill) {
                setActivity(idleCheckSkills.get(skill));
                lastExperience = Instant.now();
            }
        }
    }

    @Subscribe
    @SuppressWarnings("unused")
    private void onGameStateChanged(GameStateChanged event) {
        if (event.getGameState() == GameState.LOGGED_IN) {
            lastLogin = Instant.now();
        }
    }

    protected void setActivity(Activity action) {
        currentActivity = action;

        if (action != Activity.IDLE) {
            lastActionTime = Instant.now();
        }
    }

    protected void checkActionTimeout() {
        if (currentActivity == Activity.IDLE) return;

        if (Duration.between(lastExperience, Instant.now()).getSeconds() < 5) return;

        int animId = game.localPlayer().animation();
        if (animId != IDLE || lastActionTime == null) return;

        Duration timeout = Duration.ofSeconds(5);
        Duration sinceAction = Duration.between(lastActionTime, Instant.now());

        if (sinceAction.compareTo(timeout) >= 0) {
            setActivity(Activity.IDLE);
        }
    }

    public final boolean isLoggedIn() {
        return game.client() != null && game.client().getGameState() == GameState.LOGGED_IN;
    }

    public final boolean isLoggedInForLongerThan(int seconds) {
        return Duration.between(lastLogin, Instant.now()).getSeconds() >= seconds;
    }
}
