package io.reisub.openosrs.woodcutter;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Task;
import io.reisub.openosrs.util.Util;
import io.reisub.openosrs.woodcutter.tasks.Chop;
import io.reisub.openosrs.woodcutter.tasks.Drop;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.Client;
import net.runelite.api.events.ConfigButtonClicked;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.iUtils;
import net.runelite.client.plugins.iutils.scripts.iScript;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Woodcutter",
	description = "Cuts wood",
	enabledByDefault = false
)
@Slf4j
public class WoodcutterPlugin extends iScript {
	@Inject
	private Client client;

	private List<Task> tasks;

	@Provides
	WoodcutterConfig provideConfig(ConfigManager configManager) {
		return configManager.getConfig(WoodcutterConfig.class);
	}

	@Override
	protected void loop() {
		for (Task t : tasks) {
			if (t.validate()) {
				t.execute();
			}
		}

		game.tickDelay();
	}

	@Override
	protected void onStart() {
		log.info("Starting Chaos Woodcutter");

		tasks = new ArrayList<>();
		tasks.add(injector.getInstance(Chop.class));
		tasks.add(injector.getInstance(Drop.class));
	}

	@Override
	protected void onStop() {
		log.info("Stopping Chaos Woodcutter");
		tasks.clear();
	}

	@Subscribe
	private void onConfigButtonPressed(ConfigButtonClicked configButtonClicked) {
		if (configButtonClicked.getKey().equals("startButton")) {
			execute();
		}
	}
}